<?php

namespace TDevAgency\FilesManager\Helpers;

use Closure;
use Illuminate\Cache\TaggableStore;
use Illuminate\Cache\TaggedCache;
use Illuminate\Support\Facades\Cache;
use Illuminate\Support\Facades\Config;
use TDevAgency\FilesManager\ResourceModels\TdfmFile;

class CacheHelper
{
    private const WIDTH_PATTERN_CACHE_KEY = 'tdfm_file_by_id_%s_width_%s';
    private const HEIGHT_PATTERN_CACHE_KEY = 'tdfm_file_by_id_%s_height_%s';
    private const ID_PATTERN_CACHE_KEY = 'tdfm_file_by_id_%s';

    /**  @var Cache */
    private $cache;

    public function __construct(Cache $cache)
    {
        $this->cache = $cache;
    }

    public function rememberSideSize(int $id, Closure $closure, ?string $size = null, string $side = 'width')
    {
        if (is_null($size)) {
            $size = 'original';
        }
        if ($this->isTagSupport()) {
            return $this->getTag($id)->rememberForever($side . ':' . $id . ':' . $size, $closure);
        }

        return $this->cache::rememberForever(
            sprintf($side === 'width' ? self::WIDTH_PATTERN_CACHE_KEY : self::HEIGHT_PATTERN_CACHE_KEY, $id, $size),
            $closure
        );
    }

    private function isTagSupport(): bool
    {
        return $this->cache::getStore() instanceof TaggableStore;
    }

    private function getTag(int $id): TaggedCache
    {
        return $this->cache::tags('files-manager:' . $id);
    }

    public function getFileData(int $id): ?TdfmFile
    {
        if ($this->isTagSupport()) {
            return $this->getTag($id)->rememberForever(md5($id), function () use ($id) {
                return TdfmFile::findOrFail($id);
            });
        }

        return $this->cache::rememberForever('tdfm_file_by_id_' . $id, static function () use ($id) {
            return TdfmFile::findOrFail($id);
        });
    }

    public function removeFileData(int $id): bool
    {
        if ($this->isTagSupport()) {
            return $this->cache::tags('files-manager:' . $id)->flush();
        }

        $this->cache::forget(sprintf(self::ID_PATTERN_CACHE_KEY, $id));
        foreach (Config::get('tdfm.sizes') as $size => $value) {
            $this->cache::forget(sprintf(self::WIDTH_PATTERN_CACHE_KEY, $id, $size));
            $this->cache::forget(sprintf(self::HEIGHT_PATTERN_CACHE_KEY, $id, $size));
        }
        return true;
    }
}
